@@ -75,7 +75,7 @@ pub enum VNode<'src> {
     ///     Example {}
     /// }
     /// ```
-    Fragment(VFragment<'src>),
+    Fragment(&'src VFragment<'src>),
 
     /// Component nodes represent a mounted component with props, children, and a key.
     ///
@@ -156,15 +156,12 @@ impl<'src> VNode<'src> {
 
     // Create an "owned" version of the vnode.
     pub fn decouple(&self) -> VNode<'src> {
-        match self {
-            VNode::Text(t) => VNode::Text(*t),
-            VNode::Element(e) => VNode::Element(*e),
-            VNode::Component(c) => VNode::Component(*c),
-            VNode::Placeholder(a) => VNode::Placeholder(*a),
-            VNode::Fragment(f) => VNode::Fragment(VFragment {
-                children: f.children,
-                key: f.key,
-            }),
+        match *self {
+            VNode::Text(t) => VNode::Text(t),
+            VNode::Element(e) => VNode::Element(e),
+            VNode::Component(c) => VNode::Component(c),
+            VNode::Placeholder(a) => VNode::Placeholder(a),
+            VNode::Fragment(f) => VNode::Fragment(f),
         }
     }
 }
@@ -319,29 +316,59 @@ pub struct Listener<'bump> {
     pub event: &'static str,
 
     /// The actual callback that the user specified
-    pub(crate) callback: EventHandler<'bump>,
+    pub(crate) callback: InternalHandler<'bump>,
 }
 
-/// The callback based into element event listeners.
-pub struct EventHandler<'bump> {
-    pub callback: &'bump RefCell<Option<ListenerCallback<'bump>>>,
+pub type InternalHandler<'bump> = &'bump RefCell<Option<InternalListenerCallback<'bump>>>;
+type InternalListenerCallback<'bump> = BumpBox<'bump, dyn FnMut(AnyEvent) + 'bump>;
+
+type ExternalListenerCallback<'bump, T> = BumpBox<'bump, dyn FnMut(T) + 'bump>;
+
+/// The callback type generated by the `rsx!` macro when an `on` field is specified for components.
+///
+/// This makes it possible to pass `move |evt| {}` style closures into components as property fields.
+///
+///
+/// # Example
+///
+/// ```rust, ignore
+///
+/// rsx!{
+///     MyComponent { onclick: move |evt| log::info!("clicked"), }
+/// }
+///
+/// #[derive(Props)]
+/// struct MyProps<'a> {
+///     onclick: EventHandler<'a, MouseEvent>,
+/// }
+///
+/// fn MyComponent(cx: Scope<'a, MyProps<'a>>) -> Element {
+///     cx.render(rsx!{
+///         button {
+///             onclick: move |evt| cx.props.onclick.call(evt),
+///         }
+///     })
+/// }
+///
+/// ```
+pub struct EventHandler<'bump, T = ()> {
+    pub callback: &'bump RefCell<Option<ExternalListenerCallback<'bump, T>>>,
 }
 
-impl EventHandler<'_> {
-    pub fn call(&self, event: AnyEvent) {
+impl<T> EventHandler<'_, T> {
+    pub fn call(&self, event: T) {
         if let Some(callback) = self.callback.borrow_mut().as_mut() {
             callback(event);
         }
     }
+
     pub fn release(&self) {
         self.callback.replace(None);
     }
 }
 
-type ListenerCallback<'bump> = BumpBox<'bump, dyn FnMut(AnyEvent) + 'bump>;
-
-impl Copy for EventHandler<'_> {}
-impl Clone for EventHandler<'_> {
+impl<T> Copy for EventHandler<'_, T> {}
+impl<T> Clone for EventHandler<'_, T> {
     fn clone(&self) -> Self {
         Self {
             callback: self.callback,
@@ -353,6 +380,7 @@ impl Clone for EventHandler<'_> {
 /// Only supports the functional syntax
 pub struct VComponent<'src> {
     pub key: Option<&'src str>,
+    pub originator: ScopeId,
     pub scope: Cell<Option<ScopeId>>,
     pub can_memoize: bool,
     pub user_fc: *const (),
@@ -398,12 +426,16 @@ impl<P> AnyProps for VComponentProps<P> {
 /// This struct adds metadata to the final VNode about listeners, attributes, and children
 #[derive(Copy, Clone)]
 pub struct NodeFactory<'a> {
+    pub(crate) scope: &'a ScopeState,
     pub(crate) bump: &'a Bump,
 }
 
 impl<'a> NodeFactory<'a> {
-    pub fn new(bump: &'a Bump) -> NodeFactory<'a> {
-        NodeFactory { bump }
+    pub fn new(scope: &'a ScopeState) -> NodeFactory<'a> {
+        NodeFactory {
+            scope,
+            bump: &scope.wip_frame().bump,
+        }
     }
 
     #[inline]
@@ -447,19 +479,14 @@ impl<'a> NodeFactory<'a> {
         }))
     }
 
-    pub fn element<L, A, V>(
+    pub fn element(
         &self,
         el: impl DioxusElement,
-        listeners: L,
-        attributes: A,
-        children: V,
+        listeners: &'a [Listener<'a>],
+        attributes: &'a [Attribute<'a>],
+        children: &'a [VNode<'a>],
         key: Option<Arguments>,
-    ) -> VNode<'a>
-    where
-        L: 'a + AsRef<[Listener<'a>]>,
-        A: 'a + AsRef<[Attribute<'a>]>,
-        V: 'a + AsRef<[VNode<'a>]>,
-    {
+    ) -> VNode<'a> {
         self.raw_element(
             el.tag_name(),
             el.namespace(),
@@ -470,31 +497,23 @@ impl<'a> NodeFactory<'a> {
         )
     }
 
-    pub fn raw_element<L, A, V>(
+    pub fn raw_element(
         &self,
         tag_name: &'static str,
         namespace: Option<&'static str>,
-        listeners: L,
-        attributes: A,
-        children: V,
+        listeners: &'a [Listener<'a>],
+        attributes: &'a [Attribute<'a>],
+        children: &'a [VNode<'a>],
         key: Option<Arguments>,
-    ) -> VNode<'a>
-    where
-        L: 'a + AsRef<[Listener<'a>]>,
-        A: 'a + AsRef<[Attribute<'a>]>,
-        V: 'a + AsRef<[VNode<'a>]>,
-    {
-        let listeners: &'a L = self.bump.alloc(listeners);
-        let listeners = listeners.as_ref();
-
-        let attributes: &'a A = self.bump.alloc(attributes);
-        let attributes = attributes.as_ref();
-
-        let children: &'a V = self.bump.alloc(children);
-        let children = children.as_ref();
-
+    ) -> VNode<'a> {
         let key = key.map(|f| self.raw_text(f).0);
 
+        let mut items = self.scope.items.borrow_mut();
+        for listener in listeners {
+            let long_listener = unsafe { std::mem::transmute(listener) };
+            items.listeners.push(long_listener);
+        }
+
         VNode::Element(self.bump.alloc(VElement {
             tag: tag_name,
             key,
@@ -533,11 +552,12 @@ impl<'a> NodeFactory<'a> {
     where
         P: Properties + 'a,
     {
-        VNode::Component(self.bump.alloc(VComponent {
+        let vcomp = self.bump.alloc(VComponent {
             key: key.map(|f| self.raw_text(f).0),
             scope: Default::default(),
             can_memoize: P::IS_STATIC,
             user_fc: component as *const (),
+            originator: self.scope.scope_id(),
             props: RefCell::new(Some(Box::new(VComponentProps {
                 // local_props: RefCell::new(Some(props)),
                 // heap_props: RefCell::new(None),
@@ -550,10 +570,18 @@ impl<'a> NodeFactory<'a> {
                 // the transformation from this specific lifetime to the for<'a> lifetime
                 render_fn: unsafe { std::mem::transmute(component) },
             }))),
-        }))
+        });
+
+        if !P::IS_STATIC {
+            let vcomp = &*vcomp;
+            let vcomp = unsafe { std::mem::transmute(vcomp) };
+            self.scope.items.borrow_mut().borrowed_props.push(vcomp);
+        }
+
+        VNode::Component(vcomp)
     }
 
-    pub fn listener(self, event: &'static str, callback: EventHandler<'a>) -> Listener<'a> {
+    pub fn listener(self, event: &'static str, callback: InternalHandler<'a>) -> Listener<'a> {
         Listener {
             event,
             mounted_node: Cell::new(None),
@@ -574,10 +602,10 @@ impl<'a> NodeFactory<'a> {
         if nodes.is_empty() {
             VNode::Placeholder(self.bump.alloc(VPlaceholder { id: empty_cell() }))
         } else {
-            VNode::Fragment(VFragment {
+            VNode::Fragment(self.bump.alloc(VFragment {
                 children: nodes.into_bump_slice(),
                 key: None,
-            })
+            }))
         }
     }
 
@@ -613,10 +641,10 @@ impl<'a> NodeFactory<'a> {
                 );
             }
 
-            VNode::Fragment(VFragment {
+            VNode::Fragment(self.bump.alloc(VFragment {
                 children,
                 key: None,
-            })
+            }))
         }
     }
 
@@ -639,12 +667,19 @@ impl<'a> NodeFactory<'a> {
         } else {
             let children = nodes.into_bump_slice();
 
-            Some(VNode::Fragment(VFragment {
+            Some(VNode::Fragment(self.bump.alloc(VFragment {
                 children,
                 key: None,
-            }))
+            })))
         }
     }
+
+    pub fn event_handler<T>(self, f: impl FnMut(T) + 'a) -> EventHandler<'a, T> {
+        let handler: &mut dyn FnMut(T) = self.bump.alloc(f);
+        let caller = unsafe { BumpBox::from_raw(handler as *mut dyn FnMut(T)) };
+        let callback = self.bump.alloc(RefCell::new(Some(caller)));
+        EventHandler { callback }
+    }
 }
 
 impl Debug for NodeFactory<'_> {
@@ -752,17 +787,15 @@ impl IntoVNode<'_> for Arguments<'_> {
 
 impl<'a> IntoVNode<'a> for &Option<VNode<'a>> {
     fn into_vnode(self, cx: NodeFactory<'a>) -> VNode<'a> {
-        let r = self.as_ref().map(|f| f.decouple());
-        cx.fragment_from_iter(r)
+        self.as_ref()
+            .map(|f| f.into_vnode(cx))
+            .unwrap_or_else(|| cx.fragment_from_iter(None as Option<VNode>))
     }
 }
 
 impl<'a> IntoVNode<'a> for &VNode<'a> {
     fn into_vnode(self, _cx: NodeFactory<'a>) -> VNode<'a> {
+        // borrowed nodes are strange
         self.decouple()
     }
 }
-
-trait IntoAcceptedVnode<'a> {
-    fn into_accepted_vnode(self, cx: NodeFactory<'a>) -> VNode<'a>;
-}
